﻿using CaseAccountingContracts.BindingModels;
using CaseAccountingContracts.BusinessLogicContracts;
using CaseAccountingContracts.SearchModels;
using CaseAccountingContracts.StoragesContracts;
using CaseAccountingContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CaseAccountingBusinessLogic.BusinessLogics
{
    public class CaseLogic : ICaseLogic
    {
        private readonly ILogger _logger;
        private readonly ICaseStorage _caseStorage;

        public CaseLogic(ILogger<CaseLogic> logger, ICaseStorage caseStorage)
        {
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
            _caseStorage = caseStorage ?? throw new ArgumentNullException(nameof(logger));
        }

        public bool Create(CaseBindingModel model)
        {
            CheckModel(model);
            if (_caseStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(CaseBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_caseStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public CaseViewModel? ReadElement(CaseSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. CaseName:{Name}.Id:{ Id}", model.Name, model.Id);
            var element = _caseStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<CaseViewModel>? ReadList(CaseSearchModel? model)
        {
            _logger.LogInformation("ReadList. CaseName:{Name}.Id:{ Id}", model?.Name, model?.Id);
            var list = model == null ? _caseStorage.GetFullList() : _caseStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public List<SpecializationCasesViewModel> GetSpecializationCases()
        {
            return _caseStorage.GetSpecializationCases() ?? new List<SpecializationCasesViewModel>();
        }

        public bool Update(CaseBindingModel model)
        {
            CheckModel(model);
            if (_caseStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(CaseBindingModel model, bool withParams = true)
        {
            if (model == null) 
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.UserId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор пользователя",
                    nameof(model.UserId));
            }
            if (model.SpecializationId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор специализации",
                    nameof(model.SpecializationId));
            }
            if (model.Name == string.Empty)
            {
                throw new ArgumentNullException("Некорректное название дела",
                    nameof(model.Name));
            }
            if(model.Date > DateTime.Now) 
            {
                throw new ArgumentNullException("Некорректное время заведения дела",
                    nameof(model.Date));
            }
        }
    }
}
