﻿using CaseAccountingContracts.BindingModels;
using CaseAccountingContracts.BusinessLogicContracts;
using CaseAccountingContracts.SearchModels;
using CaseAccountingContracts.StoragesContracts;
using CaseAccountingContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CaseAccountingBusinessLogic.BusinessLogics
{
    public class DealLogic : IDealLogic
    {
        private readonly ILogger _logger;
        private readonly IDealStorage _dealStorage;

        public DealLogic(ILogger<DealLogic> logger, IDealStorage dealStorage)
        {
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
            _dealStorage = dealStorage ?? throw new ArgumentNullException(nameof(logger));
        }

        public bool Create(DealBindingModel model)
        {
            CheckModel(model);
            if (_dealStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(DealBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_dealStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public DealViewModel? ReadElement(DealSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _dealStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<DealViewModel>? ReadList(DealSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}", model?.Id);
            var list = model == null ? _dealStorage.GetFullList() : _dealStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool Update(DealBindingModel model)
        {
            CheckModel(model);
            if (_dealStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(DealBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.UserId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор пользователя",
                    nameof(model.UserId));
            }
            if (model.Subject == string.Empty) 
            {
                throw new ArgumentNullException("Некорректный предмет договора",
                    nameof(model.Subject));
            }
            if (model.Responsibilities == string.Empty) 
            {
                throw new ArgumentNullException("Некорректно указаны обязанности договора",
                    nameof(model.Responsibilities));
            }
        }
    }
}
